@@ -1,5 +1,5 @@
 #
-# (C) Copyright 2003-2005 Jacek Konieczny <jajcus@jajcus.net>
+# (C) Copyright 2003-2006 Jacek Konieczny <jajcus@jajcus.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License Version
@@ -20,29 +20,25 @@ Normative reference:
   - `JEP 78 <http://www.jabber.org/jeps/jep-0078.html>`__
 """
 
-__revision__="$Id: clientstream.py 616 2005-12-26 15:28:56Z jajcus $"
+__revision__="$Id: clientstream.py 683 2008-12-05 18:25:45Z jajcus $"
 __docformat__="restructuredtext en"
 
-import sha
+try:
+    import hashlib
+    sha_factory = hashlib.sha1
+except ImportError:
+    import sha
+    sha_factory = sha.new
+
 import logging
 
 from pyxmpp.iq import Iq
 from pyxmpp.utils import to_utf8,from_utf8
 from pyxmpp.jid import JID
-
-from pyxmpp.clientstream import ClientStreamError
 from pyxmpp.clientstream import ClientStream
-
 from pyxmpp.jabber.register import Register
 
-class LegacyAuthenticationError(ClientStreamError):
-    """Raised on a legacy authentication error."""
-    pass
-
-class RegistrationError(ClientStreamError):
-    """Raised on a in-band registration error."""
-    pass
-
+from pyxmpp.exceptions import ClientStreamError, LegacyAuthenticationError, RegistrationError
 
 class LegacyClientStream(ClientStream):
     """Handles Jabber (both XMPP and legacy protocol) client connection stream.
@@ -50,9 +46,9 @@ class LegacyClientStream(ClientStream):
     Both client and server side of the connection is supported. This class handles
     client SASL and legacy authentication, authorisation and XMPP resource binding.
     """
-    def __init__(self,jid,password=None,server=None,port=5222,
-            auth_methods=("sasl:DIGEST-MD5","digest"),
-            tls_settings=None,keepalive=0):
+    def __init__(self, jid, password = None, server = None, port = 5222,
+            auth_methods = ("sasl:DIGEST-MD5", "digest"),
+            tls_settings = None, keepalive = 0, owner = None):
         """Initialize a LegacyClientStream object.
 
         :Parameters:
@@ -64,6 +60,7 @@ class LegacyClientStream(ClientStream):
               in the list should be prefixed with "sasl:" string.
             - `tls_settings`: settings for StartTLS -- `TLSSettings` instance.
             - `keepalive`: keepalive output interval. 0 to disable.
+            - `owner`: `Client`, `Component` or similar object "owning" this stream.
         :Types:
             - `jid`: `pyxmpp.JID`
             - `password`: `unicode`
@@ -76,8 +73,8 @@ class LegacyClientStream(ClientStream):
         (self.authenticated, self.available_auth_methods, self.auth_stanza,
                 self.peer_authenticated, self.auth_method_used,
                 self.registration_callback, self.registration_form, self.__register) = (None,) * 8
-        ClientStream.__init__(self,jid,password,server,port,
-                            auth_methods,tls_settings,keepalive)
+        ClientStream.__init__(self, jid, password, server, port,
+                            auth_methods, tls_settings, keepalive, owner)
         self.__logger=logging.getLogger("pyxmpp.jabber.LegacyClientStream")
 
     def _reset(self):
@@ -324,7 +321,9 @@ class LegacyClientStream(ClientStream):
         q=iq.new_query("jabber:iq:auth")
         q.newTextChild(None,"username",to_utf8(self.my_jid.node))
         q.newTextChild(None,"resource",to_utf8(self.my_jid.resource))
-        digest=sha.new(to_utf8(self.stream_id)+to_utf8(self.password)).hexdigest()
+
+        digest = sha_factory(to_utf8(self.stream_id)+to_utf8(self.password)).hexdigest()
+
         q.newTextChild(None,"digest",digest)
         self.send(iq)
         self.set_response_handlers(iq,self.auth_finish,self.auth_error)
@@ -352,7 +351,7 @@ class LegacyClientStream(ClientStream):
             self.send(iq)
             return
 
-        mydigest=sha.new(to_utf8(self.stream_id)+to_utf8(password)).hexdigest()
+        mydigest = sha_factory(to_utf8(self.stream_id)+to_utf8(password)).hexdigest()
 
         if mydigest==digest:
             iq=stanza.make_result_response()
@@ -466,7 +465,7 @@ class LegacyClientStream(ClientStream):
                     self.my_jid = JID(self.registration_form['username'].value,
                             self.my_jid.domain, self.my_jid.resource)
                 if 'password' in self.registration_form:
-                    self.password = self.registration_form['password']
+                    self.password = self.registration_form['password'].value
             self.registration_callback = None
             self._post_connect()
         finally:
