@@ -1,5 +1,5 @@
 #
-# (C) Copyright 2003-2005 Jacek Konieczny <jajcus@jajcus.net>
+# (C) Copyright 2003-2006 Jacek Konieczny <jajcus@jajcus.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License Version
@@ -22,7 +22,7 @@ Normative reference:
   - `RFC 3920 <http://www.ietf.org/rfc/rfc3920.txt>`__
 """
 
-__revision__="$Id: streambase.py 603 2005-10-14 16:06:24Z jajcus $"
+__revision__="$Id: streambase.py 652 2006-08-27 19:41:15Z jajcus $"
 __docformat__="restructuredtext en"
 
 import libxml2
@@ -38,7 +38,7 @@ import logging
 from pyxmpp import xmlextra
 from pyxmpp.expdict import ExpiringDictionary
 from pyxmpp.utils import to_utf8
-from pyxmpp.stanza import Stanza,StanzaError
+from pyxmpp.stanza import Stanza
 from pyxmpp.error import StreamErrorNode
 from pyxmpp.iq import Iq
 from pyxmpp.presence import Presence
@@ -46,46 +46,22 @@ from pyxmpp.message import Message
 from pyxmpp.jid import JID
 from pyxmpp import resolver
 from pyxmpp.stanzaprocessor import StanzaProcessor
+from pyxmpp.exceptions import StreamError, StreamEncryptionRequired, HostMismatch, ProtocolError
+from pyxmpp.exceptions import FatalStreamError, StreamParseError, StreamAuthenticationError
 
 STREAM_NS="http://etherx.jabber.org/streams"
 BIND_NS="urn:ietf:params:xml:ns:xmpp-bind"
 
-class StreamError(StandardError):
-    """Base class for all stream errors."""
-    pass
-
-class StreamEncryptionRequired(StreamError):
-    """Exception raised when stream encryption is requested, but not used."""
-    pass
-
-class HostMismatch(StreamError):
-    """Exception raised when the connected host name is other then requested."""
-    pass
-
-class FatalStreamError(StreamError):
-    """Base class for all fatal Stream exceptions.
-
-    When `FatalStreamError` is raised the stream is no longer usable."""
-    pass
-
-class StreamParseError(FatalStreamError):
-    """Raised when invalid XML is received in an XMPP stream."""
-    pass
-
-class StreamAuthenticationError(FatalStreamError):
-    """Raised when stream authentication fails."""
-    pass
-
-def stanza_factory(xmlnode):
+def stanza_factory(xmlnode, stream = None):
     """Creates Iq, Message or Presence object for XML stanza `xmlnode`"""
     if xmlnode.name=="iq":
-        return Iq(xmlnode)
+        return Iq(xmlnode, stream = stream)
     if xmlnode.name=="message":
-        return Message(xmlnode)
+        return Message(xmlnode, stream = stream)
     if xmlnode.name=="presence":
-        return Presence(xmlnode)
+        return Presence(xmlnode, stream = stream)
     else:
-        return Stanza(xmlnode)
+        return Stanza(xmlnode, stream = stream)
 
 class StreamBase(StanzaProcessor,xmlextra.StreamHandler):
     """Base class for a generic XMPP stream.
@@ -107,9 +83,10 @@ class StreamBase(StanzaProcessor,xmlextra.StreamHandler):
         - `process_all_stanzas`: when `True` then all stanzas received are
           considered local.
         - `initiator`: `True` if local stream endpoint is the initiating entity.
+        - `owner`: `Client`, `Component` or similar object "owning" this stream.
         - `_reader`: the stream reader object (push parser) for the stream.
     """
-    def __init__(self,default_ns,extra_ns=(),keepalive=0):
+    def __init__(self, default_ns, extra_ns = (), keepalive = 0, owner = None):
         """Initialize Stream object
 
         :Parameters:
@@ -118,6 +95,7 @@ class StreamBase(StanzaProcessor,xmlextra.StreamHandler):
           - `extra_ns`: sequence of extra namespace URIs to be defined for
             the stream.
           - `keepalive`: keepalive output interval. 0 to disable.
+          - `owner`: `Client`, `Component` or similar object "owning" this stream.
         """
         StanzaProcessor.__init__(self)
         xmlextra.StreamHandler.__init__(self)
@@ -131,6 +109,7 @@ class StreamBase(StanzaProcessor,xmlextra.StreamHandler):
         self.process_all_stanzas=False
         self.port=None
         self._reset()
+        self.owner = owner
         self.__logger=logging.getLogger("pyxmpp.Stream")
 
     def _reset(self):
@@ -525,7 +504,7 @@ class StreamBase(StanzaProcessor,xmlextra.StreamHandler):
         logging.getLogger("pyxmpp.Stream.out").debug("OUT: %r",data)
         try:
             self.socket.send(data)
-        except (IOError,OSError),e:
+        except (IOError,OSError,socket.error),e:
             raise FatalStreamError("IO Error: "+str(e))
 
     def _write_node(self,xmlnode):
@@ -564,9 +543,9 @@ class StreamBase(StanzaProcessor,xmlextra.StreamHandler):
         """Same as `Stream.send` but assume `self.lock` is acquired."""
         if not self.version:
             try:
-                err=stanza.get_error()
-            except StanzaError:
-                err=None
+                err = stanza.get_error()
+            except ProtocolError:
+                err = None
             if err:
                 err.downgrade()
         self.fix_out_stanza(stanza)
@@ -661,7 +640,7 @@ class StreamBase(StanzaProcessor,xmlextra.StreamHandler):
                 raise StreamParseError(unicode(e))
             except:
                 raise
-        except (IOError,OSError),e:
+        except (IOError,OSError,socket.error),e:
             self.close()
             raise FatalStreamError("IO Error: "+str(e))
         except (FatalStreamError,KeyboardInterrupt,SystemExit),e:
@@ -725,7 +704,7 @@ class StreamBase(StanzaProcessor,xmlextra.StreamHandler):
             return
 
         if ns_uri==self.default_ns_uri:
-            stanza=stanza_factory(xmlnode)
+            stanza=stanza_factory(xmlnode, self)
             self.lock.release()
             try:
                 self.process_stanza(stanza)
@@ -845,7 +824,7 @@ class StreamBase(StanzaProcessor,xmlextra.StreamHandler):
         Set `self.me` to the full JID negotiated."""
         jid_n=stanza.xpath_eval("bind:bind/bind:jid",{"bind":BIND_NS})
         if jid_n:
-            self.me=JID(jid_n[0].getContent())
+            self.me=JID(jid_n[0].getContent().decode("utf-8"))
         self.state_change("authorized",self.me)
 
     def _bind_error(self,stanza):
