@@ -1,5 +1,5 @@
 #
-# (C) Copyright 2003-2005 Jacek Konieczny <jajcus@jajcus.net>
+# (C) Copyright 2003-2006 Jacek Konieczny <jajcus@jajcus.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License Version
@@ -24,7 +24,7 @@ Normative reference:
 """
 
 
-__revision__="$Id: vcard.py 616 2005-12-26 15:28:56Z jajcus $"
+__revision__="$Id: vcard.py 675 2007-06-28 08:36:17Z jajcus $"
 __docformat__="restructuredtext en"
 
 import base64
@@ -36,6 +36,7 @@ import pyxmpp.jid
 from pyxmpp.utils import to_utf8,from_utf8
 from pyxmpp.xmlextra import get_node_ns
 from pyxmpp.objects import StanzaPayloadObject
+from pyxmpp.exceptions import BadRequestProtocolError, JIDMalformedProtocolError, JIDError
 
 VCARD_NS="vcard-temp"
 
@@ -203,7 +204,10 @@ class VCardJID(VCardField):
         _unused = rfc2425parameters
         VCardField.__init__(self,name)
         if isinstance(value,libxml2.xmlNode):
-            self.value=pyxmpp.jid.JID(value.getContent())
+            try:
+                self.value=pyxmpp.jid.JID(value.getContent())
+            except JIDError:
+                raise JIDMalformedProtocolError, "JID malformed"
         else:
             self.value=pyxmpp.jid.JID(value)
         if not self.value:
@@ -292,7 +296,7 @@ class VCardName(VCardField):
                     empty=0
                 n=n.next
             if empty:
-                raise Empty,"Empty N value"
+                raise Empty, "Empty N value"
         else:
             v=value.split(";")
             value=[u""]*5
@@ -363,8 +367,8 @@ class VCardImage(VCardField):
         VCardField.__init__(self,name)
         if not rfc2425parameters:
             rfc2425parameters={}
+        self.uri,self.type,self.image=[None]*3
         if isinstance(value,libxml2.xmlNode):
-            self.uri,self.type,self.image=[None]*3
             n=value.children
             vns=get_node_ns(value)
             while n:
@@ -387,7 +391,7 @@ class VCardImage(VCardField):
             if (not self.uri and not self.image):
                 raise Empty,"Bad %s value in vcard" % (name,)
         else:
-            if rfc2425parameters.get("value").lower()=="uri":
+            if rfc2425parameters.get("value", "").lower()=="uri":
                 self.uri=value
                 self.type=None
             else:
@@ -482,8 +486,8 @@ class VCardAdr(VCardField):
                 self.type=t.split(",")
             else:
                 self.type=["intl","postal","parcel","work"]
-            value=[""]*7
             v=value.split(";")
+            value=[""]*7
             value[:len(v)]=v
             (self.pobox,self.extadr,self.street,self.locality,
                     self.region,self.pcode,self.ctry)=value
@@ -1038,8 +1042,8 @@ class VCardSound(VCardField):
         VCardField.__init__(self,name)
         if not rfc2425parameters:
             rfc2425parameters={}
+        self.uri,self.sound,self.phonetic=[None]*3
         if isinstance(value,libxml2.xmlNode):
-            self.uri,self.sound,self.phonetic=[None]*3
             n=value.children
             vns=get_node_ns(value)
             while n:
@@ -1066,7 +1070,7 @@ class VCardSound(VCardField):
             if (not self.phonetic and not self.uri and not self.sound):
                 raise Empty,"Bad SOUND value in vcard"
         else:
-            if rfc2425parameters.get("value").lower()=="uri":
+            if rfc2425parameters.get("value", "").lower()=="uri":
                 self.uri=value
                 self.sound=None
                 self.phonetic=None
@@ -1302,7 +1306,7 @@ class VCard(StanzaPayloadObject):
         - `desc`: `list` of `VCardXString`
     """
 
-    xml_element_name = "query"
+    xml_element_name = "vCard"
     xml_element_namespace = VCARD_NS
 
     components={
@@ -1406,9 +1410,9 @@ class VCard(StanzaPayloadObject):
             - `data`: `libxml2.xmlNode`"""
         ns=get_node_ns(data)
         if ns and ns.getContent()!=VCARD_NS:
-            raise ValueError,"Not in the %r namespace" % (VCARD_NS,)
+            raise ValueError, "Not in the %r namespace" % (VCARD_NS,)
         if data.name!="vCard":
-            raise ValueError,"Bad root element name: %r" % (data.name,)
+            raise ValueError, "Bad root element name: %r" % (data.name,)
         n=data.children
         dns=get_node_ns(data)
         while n:
